import logging
import operator
import re

from django.conf import settings
from django.utils.cache import patch_vary_headers

from subdomains.utils import get_domain

logger = logging.getLogger(__name__)
lower = operator.methodcaller("lower")

UNSET = object()


class SubdomainURLRoutingMiddleware:
    """
    A middleware class that allows for subdomain-based URL routing.
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        self.process_request_subdomain(request)
        self.process_request_urlconf(request)
        response = self.get_response(request)
        self.process_response(response)
        return response

    @staticmethod
    def process_request_subdomain(request):
        domain, host = map(lower, (get_domain(), request.get_host()))

        pattern = r"^(?:(?P<subdomain>.*?)\.)?%s(?::.*)?$" % re.escape(domain)
        matches = re.match(pattern, host)

        if matches:
            request.subdomain = matches.group("subdomain")
        else:
            request.subdomain = None
            logger.warning(
                "The host {host} does not belong to the domain {domain}, "
                "unable to identify the subdomain for this request".format(
                    host=request.get_host(),
                    domain=domain,
                )
            )

    @staticmethod
    def process_request_urlconf(request):
        """
        Sets the current request's ``urlconf`` attribute to the urlconf
        associated with the subdomain, if it is listed in
        ``settings.SUBDOMAIN_URLCONFS``.
        """
        SubdomainURLRoutingMiddleware.process_request_subdomain(request)
        subdomain = getattr(request, "subdomain", UNSET)

        if subdomain is not UNSET:
            urlconf = settings.SUBDOMAIN_URLCONFS.get(subdomain)
            if urlconf is not None:
                logger.debug(f"Using urlconf {repr(urlconf)} for subdomain: {repr(subdomain)}")
                request.urlconf = urlconf

    @staticmethod
    def process_response(response):
        """
        Forces the HTTP ``Vary`` header onto requests to avoid having responses
        cached across subdomains.
        """
        if getattr(settings, "FORCE_VARY_ON_HOST", True):
            patch_vary_headers(response, ("Host",))
        return



import functools
from urllib.parse import urlunparse

from django.conf import settings
from django.urls import reverse as simple_reverse


def current_site_domain():
    from django.contrib.sites.models import Site

    domain = Site.objects.get_current().domain

    prefix = "www."
    if getattr(settings, "REMOVE_WWW_FROM_DOMAIN", False) and domain.startswith(prefix):
        domain = domain.replace(prefix, "", 1)

    return domain


get_domain = current_site_domain


def urljoin(domain, path=None, scheme=None):
    """
    Joins a domain, path and scheme part together, returning a full URL.

    :param domain: the domain, e.g. ``example.com``
    :param path: the path part of the URL, e.g. ``/example/``
    :param scheme: the scheme part of the URL, e.g. ``http``, defaulting to the
        value of ``settings.DEFAULT_URL_SCHEME``
    :returns: a full URL
    """
    if scheme is None:
        scheme = getattr(settings, "DEFAULT_URL_SCHEME", "http")

    return urlunparse((scheme, domain, path or "", None, None, None))


def reverse(viewname, subdomain=None, scheme=None, args=None, kwargs=None, current_app=None):
    """
    Reverses a URL from the given parameters, in a similar fashion to
    :meth:`django.core.urlresolvers.reverse`.

    :param viewname: the name of URL
    :param subdomain: the subdomain to use for URL reversing
    :param scheme: the scheme to use when generating the full URL
    :param args: positional arguments used for URL reversing
    :param kwargs: named arguments used for URL reversing
    :param current_app: hint for the currently executing application
    """
    urlconf = settings.SUBDOMAIN_URLCONFS.get(subdomain, settings.ROOT_URLCONF)

    domain = get_domain()
    if subdomain is not None:
        domain = f"{subdomain}.{domain}"

    path = simple_reverse(viewname, urlconf=urlconf, args=args, kwargs=kwargs, current_app=current_app)
    return urljoin(domain, path, scheme=scheme)


#: :func:`reverse` bound to insecure (non-HTTPS) URLs scheme
insecure_reverse = functools.partial(reverse, scheme="http")

#: :func:`reverse` bound to secure (HTTPS) URLs scheme
secure_reverse = functools.partial(reverse, scheme="https")

#: :func:`reverse` bound to be relative to the current scheme
relative_reverse = functools.partial(reverse, scheme="")
